package com.hcordova.flightagencyapi.shared.services;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import com.hcordova.flightagencyapi.shared.domain.models.User;
import com.hcordova.flightagencyapi.shared.domain.repositories.RoleRepository;
import com.hcordova.flightagencyapi.shared.domain.repositories.UserRepository;
import com.hcordova.flightagencyapi.shared.domain.services.AuthenticationService;
import com.hcordova.flightagencyapi.shared.domain.services.JwtService;
import com.hcordova.flightagencyapi.shared.dto.SignInRequestDTO;
import com.hcordova.flightagencyapi.shared.dto.SignInResponseDTO;
import com.hcordova.flightagencyapi.shared.dto.SignUpRequestDTO;
import com.hcordova.flightagencyapi.shared.dto.SignUpResponseDTO;

import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class AuthenticationServiceImpl implements AuthenticationService {

    private final UserRepository userRepository;
    private final RoleRepository roleRepository;
    private final JwtService jService;
    private final AuthenticationManager authenticationManager;
    private final PasswordEncoder passwordEncoder;

    @Override
    public ResponseEntity<SignInResponseDTO> signIn(SignInRequestDTO requestDTO) {
        authenticationManager.authenticate(
            new UsernamePasswordAuthenticationToken(requestDTO.getEmail(), requestDTO.getPassword()));
        var user = userRepository.findByEmail(requestDTO.getEmail())
            .orElseThrow(() -> new UsernameNotFoundException("Invalid email or password"));
        var jwt = jService.generateToken(user);

        var response = new SignInResponseDTO(user.getId(), user.getName(), user.getLastname(), user.getEmail(), user.getRole().getName(), jwt);

        return ResponseEntity.status(HttpStatus.OK).body(response);
    }

    @Override
    @Transactional
    public ResponseEntity<SignUpResponseDTO> signUp(SignUpRequestDTO requestDTO) {
        var userFound = userRepository.findByEmail(requestDTO.getEmail());

        if (userFound.isPresent()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(null);
        }

        var role = roleRepository.findByName(requestDTO.getRole());

        if (role.isEmpty()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(null);
        }

        var user = new User(
            requestDTO.getName(), 
            requestDTO.getLastname(), 
            requestDTO.getEmail(), 
            passwordEncoder.encode(requestDTO.getPassword()), 
            role.get());
        
        var userCreated = userRepository.save(user);

        var response = new SignUpResponseDTO(userCreated.getId(), userCreated.getName(), userCreated.getLastname(), userCreated.getEmail(), userCreated.getRole().getName());
    
        return ResponseEntity.status(HttpStatus.CREATED).body(response);
    }

    // private boolean registerPassenger(String secondName, String country, String city, String phone, String address) {

    // }

    // private boolean registerAgent(String details) {

    // }

    // private boolean registerSupervisor(String details) {

    // }
    
}
